/**
 * Decoder
 *
 * Accepts stringifed JSON serialization, as well as msgpack binary encoding. 
 */

'use strict';

/* Requires ------------------------------------------------------------------*/

var msgpack = require('msgpack-lite');

/* Local variables -----------------------------------------------------------*/

var _token = decodeURIComponent('%EF%BF%BD');
var _reg = new RegExp(_token);

/* Methods -------------------------------------------------------------------*/

/**
 * Converts a Buffer, WS-Object or ArrayBuffer to an object.
 * Accepts JSON and MSG-Pack encoding
 * @method decode
 * @param {Buffer} buffer The buffer to decode
 */
function decode(buffer) {
	var raw;

	if (buffer instanceof ArrayBuffer || buffer.data) {
		raw = _token;
		buffer = new Uint8Array(buffer.data || buffer);
	}
	else {
		raw = buffer.toString();
	}

	//Not friendly to old node version 
	if (_reg.test(raw)) return msgpack.decode(buffer);
	else return JSON.parse(raw);
}

/* Exports -------------------------------------------------------------------*/

module.exports = decode;